/*
 * This program panics NetBSD/i386 20011117 in nathanw_sa branch
 * running in vmware 2.0.
 *
 * Start the program and press enter within a second and the
 * kernel will panic.
 */

#include <sys/types.h>

#include <err.h>
#include <poll.h>
#include <pthread.h>
#include <string.h>
#include <unistd.h>

#define WAKE_DELAY 1000

static int wake_fd;

static void *wake_thread(void *);

int
main(int argc, char *argv[])
{
	struct pollfd	pfds[2];
	pthread_t	wake_tid;
	int		fd[2], wake_read_fd, e;
	char		b[100];

	if (pipe(fd) != 0)
		err(1, "pipe");

	wake_fd = fd[1];
	wake_read_fd = fd[0];

	memset(pfds, 0, sizeof(struct pollfd) * 2);

	pfds[0].fd = wake_read_fd;
	pfds[0].events = POLLIN;
	pfds[1].fd = STDIN_FILENO;
	pfds[1].events = POLLIN;

	e = pthread_create(&wake_tid, NULL, wake_thread, "wake_thread");
	if (e != 0)
		errx(1, "pthread_create: %s", strerror(e));

	for ( ;; ) {
		warnx("main: waiting for IO");
		if (poll(pfds, 2, INFTIM) == -1)
			err(1, "poll");

		if (pfds[0].revents) {
			warnx("awakened by wake_thread, revents 0x%x",
				(int)pfds[0].revents);
			if (read(wake_read_fd, b, sizeof(b)) == -1)
				err(1, "main: read wake_read_fd");
		}
		if (pfds[1].revents) {
			warnx("got input from stdin, revents 0x%x",
				(int)pfds[1].revents);
			if (read(STDIN_FILENO, b, sizeof(b)) == -1)
				err(1, "main: read stdin");
		}
	}
	

}


static void *
wake_thread(void *arg)
{
	struct pollfd pfd;
	char	*name;
	char	b;

	name = arg;

	memset(&pfd, 0, sizeof(struct pollfd));

	for ( ;; ) {
		/*
		 * For some reason "poll(NULL, 0, WAKE_DELAY)"
		 * didn't work, but I suppose that's another issue.
		 */
		if (poll(&pfd, 1, WAKE_DELAY) == -1)
			err(1, "%s: poll", name);

		warnx("%s: wakening main", name);
		if (write(wake_fd, &b, 1) != 1)
			err(1, "%s: write", name);
	}
}
